@@ -1,6 +1,6 @@
 use std::{
     fs, io,
-    path::Path,
+    path::{Path, PathBuf},
     process::{Command, Stdio},
     sync::Arc,
 };
@@ -11,7 +11,7 @@ use once_cell::sync::Lazy;
 use regex::Regex;
 use tempfile::tempdir;
 
-use crate::{RangeExt, Workspace};
+use crate::{Document, RangeExt, Workspace};
 
 use super::{Diagnostic, DiagnosticCode};
 
@@ -28,26 +28,28 @@ pub fn collect_chktex_diagnostics(
         diagnostics
     });
 
-    let current_dir = workspace
-        .environment
-        .options
-        .root_directory
-        .as_ref()
-        .cloned()
+    let current_dir = find_chktexrc_directory(&document)
         .or_else(|| {
-            if document.uri.scheme() == "file" {
-                document
-                    .uri
-                    .to_file_path()
-                    .unwrap()
-                    .parent()
-                    .map(ToOwned::to_owned)
-            } else {
-                None
-            }
+            workspace
+                .environment
+                .options
+                .root_directory
+                .as_ref()
+                .cloned()
+        })
+        .or_else(|| {
+            workspace
+                .find_parent(uri)
+                .as_ref()
+                .or_else(|| Some(document))
+                .filter(|doc| doc.uri.scheme() == "file")
+                .and_then(|doc| doc.uri.to_file_path().ok())
+                .and_then(|path| path.parent().map(ToOwned::to_owned))
         })
         .unwrap_or_else(|| ".".into());
 
+    log::debug!("Calling ChkTeX from directory: {}", current_dir.display());
+
     all_diagnostics
         .entry(Arc::clone(&document.uri))
         .or_default()
@@ -56,16 +58,35 @@ pub fn collect_chktex_diagnostics(
     Some(())
 }
 
+static CHKTEXRC_FILES: &[&str] = &["chktexrc", ".chktexrc"];
+
+fn find_chktexrc_directory(document: &Document) -> Option<PathBuf> {
+    if document.uri.scheme() == "file" {
+        if let Ok(mut path) = document.uri.to_file_path() {
+            while path.pop() {
+                if CHKTEXRC_FILES
+                    .iter()
+                    .any(|rc_file| path.join(rc_file).exists())
+                {
+                    return Some(path);
+                }
+            }
+        }
+    }
+
+    None
+}
+
 static LINE_REGEX: Lazy<Regex> =
     Lazy::new(|| Regex::new("(\\d+):(\\d+):(\\d+):(\\w+):(\\w+):(.*)").unwrap());
 
 fn lint(text: &str, current_dir: &Path) -> io::Result<Vec<Diagnostic>> {
     let directory = tempdir()?;
     fs::write(directory.path().join("file.tex"), text)?;
-    drop(fs::copy(
-        current_dir.join("chktexrc"),
-        directory.path().join("chktexrc"),
-    ));
+
+    for rc_file in CHKTEXRC_FILES {
+        let _ = fs::copy(current_dir.join(rc_file), directory.path().join(rc_file));
+    }
 
     let output = Command::new("chktex")
         .args(&["-I0", "-f%l:%c:%d:%k:%n:%m\n", "file.tex"])
