@@ -20,7 +20,10 @@ use serde::{Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 use uuid::Uuid;
 
-use crate::{client, req_queue::ReqQueue, ClientCapabilitiesExt, DocumentLanguage};
+use crate::{
+    client::{self, ReqQueue},
+    ClientCapabilitiesExt, DocumentLanguage,
+};
 
 use super::{forward_search, FeatureRequest};
 
@@ -165,11 +168,12 @@ impl BuildEngine {
         let args: Vec<_> = options
             .build
             .args
+            .0
             .iter()
             .map(|arg| replace_placeholder(arg.clone(), &path))
             .collect();
 
-        let mut process = Command::new(&options.build.executable)
+        let mut process = Command::new(&options.build.executable.0)
             .args(args)
             .stdin(Stdio::null())
             .stdout(Stdio::piped())
