@@ -1,7 +1,6 @@
 use std::{
     path::PathBuf,
     sync::{Arc, Mutex},
-    time::Duration,
 };
 
 use anyhow::Result;
@@ -9,11 +8,14 @@ use crossbeam_channel::{Receiver, Sender};
 use log::{error, info, warn};
 use lsp_server::{Connection, Message, RequestId};
 use lsp_types::{notification::*, request::*, *};
+use rowan::ast::AstNode;
 use serde::Serialize;
 use threadpool::ThreadPool;
 
 use crate::{
-    client::{send_notification, send_request},
+    citation,
+    client::{send_notification, send_request, ReqQueue},
+    component_db::COMPONENT_DATABASE,
     debouncer,
     diagnostics::DiagnosticManager,
     dispatch::{NotificationDispatcher, RequestDispatcher},
@@ -22,10 +24,10 @@ use crate::{
         execute_command, find_all_references, find_document_highlights, find_document_links,
         find_document_symbols, find_foldings, find_hover, find_workspace_symbols,
         format_source_code, goto_definition, prepare_rename_all, rename_all, BuildEngine,
-        BuildParams, BuildResult, BuildStatus, FeatureRequest, ForwardSearchResult,
-        ForwardSearchStatus,
+        BuildParams, BuildResult, BuildStatus, CompletionItemData, FeatureRequest,
+        ForwardSearchResult, ForwardSearchStatus,
     },
-    req_queue::{IncomingData, ReqQueue},
+    syntax::bibtex,
     ClientCapabilitiesExt, Document, DocumentData, DocumentLanguage, Environment, LineIndex,
     LineIndexExt, Options, Workspace, WorkspaceEvent,
 };
@@ -34,6 +36,7 @@ use crate::{
 enum InternalMessage {
     SetDistro(Distribution),
     SetOptions(Arc<Options>),
+    FileEvent(notify::Event),
 }
 
 #[derive(Clone)]
@@ -101,7 +104,6 @@ impl Server {
             definition_provider: Some(OneOf::Left(true)),
             references_provider: Some(OneOf::Left(true)),
             hover_provider: Some(HoverProviderCapability::Simple(true)),
-            #[cfg(feature = "completion")]
             completion_provider: Some(CompletionOptions {
                 resolve_provider: Some(true),
                 trigger_characters: Some(vec![
@@ -163,49 +165,26 @@ impl Server {
         }
 
         self.register_diagnostics_handler();
+        self.register_file_watching();
 
         self.spawn(move |server| {
             server.register_config_capability();
-            server.register_file_watching();
             let _ = server.pull_config();
         });
 
         Ok(())
     }
 
-    fn register_file_watching(&self) {
-        if self
-            .workspace
-            .environment
-            .client_capabilities
-            .has_file_watching_support()
-        {
-            let options = DidChangeWatchedFilesRegistrationOptions {
-                watchers: vec![FileSystemWatcher {
-                    glob_pattern: "**/*.{aux,log}".into(),
-                    kind: Some(WatchKind::Create | WatchKind::Change | WatchKind::Delete),
-                }],
-            };
-
-            let reg = Registration {
-                id: "build-watch".to_string(),
-                method: DidChangeWatchedFiles::METHOD.to_string(),
-                register_options: Some(serde_json::to_value(options).unwrap()),
-            };
-
-            let params = RegistrationParams {
-                registrations: vec![reg],
-            };
-
-            if let Err(why) =
-                send_request::<RegisterCapability>(&self.req_queue, &self.connection.sender, params)
-            {
-                error!(
-                    "Failed to register \"{}\" notification: {}",
-                    DidChangeWatchedFiles::METHOD,
-                    why
-                );
+    fn register_file_watching(&mut self) {
+        let tx = self.internal_tx.clone();
+        let watcher = notify::recommended_watcher(move |ev: Result<_, _>| {
+            if let Ok(ev) = ev {
+                let _ = tx.send(InternalMessage::FileEvent(ev));
             }
+        });
+
+        if let Ok(watcher) = watcher {
+            self.workspace.register_watcher(watcher);
         }
     }
 
@@ -248,9 +227,7 @@ impl Server {
                     WorkspaceEvent::Changed(workspace, document) => {
                         diagnostic_manager.push_syntax(&workspace, &document.uri);
                         let delay = workspace.environment.options.diagnostics_delay;
-                        diagnostic_tx
-                            .send(workspace, Duration::from_millis(delay))
-                            .unwrap();
+                        diagnostic_tx.send(workspace, delay.0).unwrap();
                     }
                 };
             }
@@ -259,11 +236,6 @@ impl Server {
         self.workspace.listeners.push(event_sender);
     }
 
-    fn register_incoming_request(&self, id: RequestId) {
-        let mut req_queue = self.req_queue.lock().unwrap();
-        req_queue.incoming.register(id, IncomingData);
-    }
-
     fn pull_config(&self) -> Result<()> {
         if !self
             .workspace
@@ -315,40 +287,18 @@ impl Server {
                     },
                 )?;
 
-                Options::default()
+                None
             }
         };
 
-        Ok(options)
+        Ok(options.unwrap_or_default())
     }
 
-    fn cancel(&self, params: CancelParams) -> Result<()> {
-        let id = match params.id {
-            NumberOrString::Number(id) => RequestId::from(id),
-            NumberOrString::String(id) => RequestId::from(id),
-        };
-
-        let mut req_queue = self.req_queue.lock().unwrap();
-        req_queue.incoming.complete(id);
-
+    fn cancel(&self, _params: CancelParams) -> Result<()> {
         Ok(())
     }
 
-    fn did_change_watched_files(&mut self, params: DidChangeWatchedFilesParams) -> Result<()> {
-        for change in params.changes {
-            if let Ok(path) = change.uri.to_file_path() {
-                match change.typ {
-                    FileChangeType::CREATED | FileChangeType::CHANGED => {
-                        self.workspace.reload(path)?;
-                    }
-                    FileChangeType::DELETED => {
-                        self.workspace.documents_by_uri.remove(&change.uri);
-                    }
-                    _ => {}
-                }
-            }
-        }
-
+    fn did_change_watched_files(&mut self, _params: DidChangeWatchedFilesParams) -> Result<()> {
         Ok(())
     }
 
@@ -488,7 +438,7 @@ impl Server {
             let delay = server.workspace.environment.options.diagnostics_delay;
             server
                 .diagnostic_tx
-                .send(server.workspace.clone(), Duration::from_millis(delay))
+                .send(server.workspace.clone(), delay.0)
                 .unwrap();
         });
     }
@@ -557,7 +507,6 @@ impl Server {
         Ok(())
     }
 
-    #[cfg(feature = "completion")]
     fn completion(&self, id: RequestId, params: CompletionParams) -> Result<()> {
         let uri = Arc::new(params.text_document_position.text_document.uri.clone());
 
@@ -569,15 +518,7 @@ impl Server {
         Ok(())
     }
 
-    #[cfg(feature = "completion")]
     fn completion_resolve(&self, id: RequestId, mut item: CompletionItem) -> Result<()> {
-        use rowan::ast::AstNode;
-
-        use crate::{
-            citation, component_db::COMPONENT_DATABASE, features::CompletionItemData,
-            syntax::bibtex,
-        };
-
         self.spawn(move |server| {
             match serde_json::from_value(item.data.clone().unwrap()).unwrap() {
                 CompletionItemData::Package | CompletionItemData::Class => {
@@ -586,19 +527,22 @@ impl Server {
                         .map(Documentation::MarkupContent);
                 }
                 CompletionItemData::Citation { uri, key } => {
-                    if let Some(document) = server.workspace.documents_by_uri.get(&uri) {
-                        if let Some(data) = document.data.as_bibtex() {
-                            let root = bibtex::SyntaxNode::new_root(data.green.clone());
-                            item.documentation = bibtex::Root::cast(root)
-                                .and_then(|root| root.find_entry(&key))
-                                .and_then(|entry| citation::render(&entry))
-                                .map(|value| {
-                                    Documentation::MarkupContent(MarkupContent {
-                                        kind: MarkupKind::Markdown,
-                                        value,
-                                    })
-                                });
-                        }
+                    if let Some(data) = server
+                        .workspace
+                        .documents_by_uri
+                        .get(&uri)
+                        .and_then(|document| document.data.as_bibtex())
+                    {
+                        let root = bibtex::SyntaxNode::new_root(data.green.clone());
+                        item.documentation = bibtex::Root::cast(root)
+                            .and_then(|root| root.find_entry(&key))
+                            .and_then(|entry| citation::render(&entry))
+                            .map(|value| {
+                                Documentation::MarkupContent(MarkupContent {
+                                    kind: MarkupKind::Markdown,
+                                    value,
+                                })
+                            });
                     }
                 }
                 _ => {}
@@ -753,6 +697,11 @@ impl Server {
             )?;
         }
 
+        let _ = match &self.workspace.environment.options.aux_directory {
+            Some(path) => self.workspace.watch(path),
+            None => self.workspace.watch(&PathBuf::from(".")),
+        };
+
         Ok(())
     }
 
@@ -766,7 +715,6 @@ impl Server {
                                 return Ok(());
                             }
 
-                            self.register_incoming_request(request.id.clone());
                             if let Some(response) = RequestDispatcher::new(request)
                                 .on::<DocumentLinkRequest, _>(|id, params| self.document_link(id, params))?
                                 .on::<FoldingRangeRequest, _>(|id, params| self.folding_range(id, params))?
@@ -777,12 +725,10 @@ impl Server {
                                 })?
                                 .on::<WorkspaceSymbol, _>(|id, params| self.workspace_symbols(id, params))?
                                 .on::<Completion, _>(|id, params| {
-                                    #[cfg(feature = "completion")]
                                     self.completion(id, params)?;
                                     Ok(())
                                 })?
                                 .on::<ResolveCompletionItem, _>(|id, params| {
-                                    #[cfg(feature = "completion")]
                                     self.completion_resolve(id, params)?;
                                     Ok(())
                                 })?
@@ -845,6 +791,25 @@ impl Server {
                             self.workspace.environment.options = options;
                             self.reparse_all()?;
                         }
+                        InternalMessage::FileEvent(ev) => {
+                            match ev.kind {
+                                notify::EventKind::Create(_) | notify::EventKind::Modify(_) => {
+                                    for path in ev.paths {
+                                        let _ = self.workspace.reload(path);
+                                    }
+                                }
+                                notify::EventKind::Remove(_) => {
+                                    for uri in
+                                        ev.paths.iter().flat_map(|path| Url::from_file_path(path))
+                                    {
+                                        self.workspace.documents_by_uri.remove(&uri);
+                                    }
+                                }
+                                notify::EventKind::Any
+                                | notify::EventKind::Access(_)
+                                | notify::EventKind::Other => {}
+                            };
+                        }
                     };
                 }
             };
